@@ -1,9 +1,9 @@
-/* $Id: glu.c,v 1.8 1997/10/04 01:30:20 brianp Exp $ */
+/* $Id: glu.c,v 1.16.2.5 2000/04/24 12:33:24 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  3.2
+ * Copyright (C) 1995-2000  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,41 +21,14 @@
  */
 
 
-/*
- * $Log: glu.c,v $
- * Revision 1.8  1997/10/04 01:30:20  brianp
- * changed version to 2.5
- *
- * Revision 1.7  1997/08/13 01:25:21  brianp
- * changed version string to 2.4
- *
- * Revision 1.6  1997/07/24 01:28:44  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
- *
- * Revision 1.5  1997/07/13 22:59:11  brianp
- * added const to viewport parameter of gluPickMatrix()
- *
- * Revision 1.4  1997/05/28 02:29:38  brianp
- * added support for precompiled headers (PCH), inserted APIENTRY keyword
- *
- * Revision 1.3  1997/04/12 16:19:02  brianp
- * changed version to 2.3
- *
- * Revision 1.2  1997/03/11 00:58:34  brianp
- * changed version to 2.2
- *
- * Revision 1.1  1996/09/27 01:19:39  brianp
- * Initial revision
- *
- */
-
-
 #ifdef PC_HEADER
 #include "all.h"
 #else
+#include <assert.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "gluP.h"
 #endif
 
@@ -73,7 +46,7 @@
 
 
 
-void APIENTRY gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
+void GLAPIENTRY gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
                          GLdouble centerx, GLdouble centery, GLdouble centerz,
                          GLdouble upx, GLdouble upy, GLdouble upz )
 {
@@ -143,7 +116,7 @@ void APIENTRY gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
 
 
 
-void APIENTRY gluOrtho2D( GLdouble left, GLdouble right,
+void GLAPIENTRY gluOrtho2D( GLdouble left, GLdouble right,
                           GLdouble bottom, GLdouble top )
 {
    glOrtho( left, right, bottom, top, -1.0, 1.0 );
@@ -151,7 +124,7 @@ void APIENTRY gluOrtho2D( GLdouble left, GLdouble right,
 
 
 
-void APIENTRY gluPerspective( GLdouble fovy, GLdouble aspect,
+void GLAPIENTRY gluPerspective( GLdouble fovy, GLdouble aspect,
                               GLdouble zNear, GLdouble zFar )
 {
    GLdouble xmin, xmax, ymin, ymax;
@@ -167,7 +140,7 @@ void APIENTRY gluPerspective( GLdouble fovy, GLdouble aspect,
 
 
 
-void APIENTRY gluPickMatrix( GLdouble x, GLdouble y,
+void GLAPIENTRY gluPickMatrix( GLdouble x, GLdouble y,
                              GLdouble width, GLdouble height,
                              const GLint viewport[4] )
 {
@@ -192,18 +165,17 @@ void APIENTRY gluPickMatrix( GLdouble x, GLdouble y,
 
 
 
-const GLubyte* APIENTRY gluErrorString( GLenum errorCode )
+const GLubyte* GLAPIENTRY gluErrorString( GLenum errorCode )
 {
    static char *tess_error[] = {
-      "missing gluEndPolygon",
       "missing gluBeginPolygon",
-      "misoriented contour",
-      "vertex/edge intersection",
+      "missing gluBeginContour",
+      "missing gluEndPolygon",
+      "missing gluEndContour",
       "misoriented or self-intersecting loops",
       "coincident vertices",
       "colinear vertices",
-      "intersecting edges",
-      "not coplanar contours"
+      "FIST recovery process fatal error"
    };
    static char *nurbs_error[] = {
       "spline order un-supported",
@@ -239,7 +211,7 @@ const GLubyte* APIENTRY gluErrorString( GLenum errorCode )
       "unconnected trim curves",
       "unknown knot error",
       "negative vertex count encountered",
-      "negative byte-stride encounteed",
+      "negative byte-stride encountered",
       "unknown type descriptor",
       "null control array or knot vector",
       "duplicate point on pwlcurve"
@@ -283,7 +255,7 @@ const GLubyte* APIENTRY gluErrorString( GLenum errorCode )
    else if (errorCode==GLU_INCOMPATIBLE_GL_VERSION) {
       return (GLubyte *) "incompatible GL version";
    }
-   else if (errorCode>=GLU_TESS_ERROR1 && errorCode<=GLU_TESS_ERROR9) {
+   else if (errorCode>=GLU_TESS_ERROR1 && errorCode<=GLU_TESS_ERROR8) {
       return (GLubyte *) tess_error[errorCode-GLU_TESS_ERROR1];
    }
    else if (errorCode>=GLU_NURBS_ERROR1 && errorCode<=GLU_NURBS_ERROR37) {
@@ -300,10 +272,10 @@ const GLubyte* APIENTRY gluErrorString( GLenum errorCode )
  * New in GLU 1.1
  */
 
-const GLubyte* APIENTRY gluGetString( GLenum name )
+const GLubyte* GLAPIENTRY gluGetString( GLenum name )
 {
-   static char *extensions = "";
-   static char *version = "1.1 Mesa 2.5";
+   static char *extensions = "GL_EXT_abgr";
+   static char *version = "1.2 Mesa 3.2";
 
    switch (name) {
       case GLU_EXTENSIONS:
@@ -315,3 +287,73 @@ const GLubyte* APIENTRY gluGetString( GLenum name )
    }
 }
 
+
+
+#if 0  /* gluGetProcAddressEXT not finalized yet! */
+
+#ifdef __cplusplus
+   /* for BeOS R4.5 */
+   void GLAPIENTRY (*gluGetProcAddressEXT(const GLubyte *procName))(...)
+#else
+   void (GLAPIENTRY *gluGetProcAddressEXT(const GLubyte *procName))()
+#endif
+{
+   struct proc {
+      const char *name;
+      void *address;
+   };
+   static struct proc procTable[] = {
+      { "gluGetProcAddressEXT", (void *) gluGetProcAddressEXT },  /* me! */
+
+      /* new 1.1 functions */
+      { "gluGetString", (void *) gluGetString },
+
+      /* new 1.2 functions */
+      { "gluTessBeginPolygon", (void *) gluTessBeginPolygon },
+      { "gluTessBeginContour", (void *) gluTessBeginContour },
+      { "gluTessEndContour", (void *) gluTessEndContour },
+      { "gluTessEndPolygon", (void *) gluTessEndPolygon },
+      { "gluGetTessProperty", (void *) gluGetTessProperty },
+
+      /* new 1.3 functions */
+
+      { NULL, NULL }
+   };
+   GLuint i;
+
+   for (i = 0; procTable[i].address; i++) {
+      if (strcmp((const char *) procName, procTable[i].name) == 0)
+         return (void (GLAPIENTRY *)()) procTable[i].address;
+   }
+
+   return NULL;
+}
+
+#endif
+
+
+
+/*
+ * New in GLU 1.3
+ */
+GLboolean GLAPIENTRY
+gluCheckExtension( const char *extName, const GLubyte *extString )
+{
+   assert(extName);
+   assert(extString);
+   {
+      const int len = strlen(extName);
+      const char *start = (const char *) extString;
+
+      while (1) {
+         const char *c = strstr( start, extName );
+         if (!c)
+            return GL_FALSE;
+
+         if ((c == start || c[-1] == ' ') && (c[len] == ' ' || c[len] == 0))
+            return GL_TRUE;
+
+         start = c + len;
+      }
+   }
+}
