@@ -1,4 +1,4 @@
-/* $Id: nurbsutl.c,v 1.7 1998/07/26 02:07:59 brianp Exp $ */
+/* $Id: nurbsutl.c,v 1.1.1.1 1999/08/19 00:55:42 jtg Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,12 @@
 
 /*
  * $Log: nurbsutl.c,v $
+ * Revision 1.1.1.1  1999/08/19 00:55:42  jtg
+ * Imported sources
+ *
+ * Revision 1.8  1999/06/08 00:44:51  brianp
+ * OpenStep updates (pete@ohm.york.ac.uk)
+ *
  * Revision 1.7  1998/07/26 02:07:59  brianp
  * updated for Windows compilation per Ted Jump
  *
@@ -91,7 +97,7 @@ test_knot(GLint nknots, GLfloat *knot, GLint order)
 
 static int
 /* qsort function */
-#ifdef WIN32
+#if defined(WIN32) && !defined(OPENSTEP)
 __cdecl
 #endif
 knot_sort(const void *a, const void *b)
