@@ -1,67 +1,45 @@
-/* $Id: blend.c,v 3.8 1998/08/16 14:44:56 brianp Exp $ */
+/* $Id: blend.c,v 1.7.2.1 2000/02/21 14:59:41 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.0
- * Copyright (C) 1995-1998  Brian Paul
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Version:  3.2
+ * 
+ * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
 
-/*
- * $Log: blend.c,v $
- * Revision 3.8  1998/08/16 14:44:56  brianp
- * fixed arithmetic problems in blend_transparency() (Karl Schultz)
- *
- * Revision 3.7  1998/08/05 01:34:40  brianp
- * fixed round-off problem exposed by conformance testing
- *
- * Revision 3.6  1998/07/09 03:15:22  brianp
- * include asm_mmx.h instead of asm-mmx.h
- *
- * Revision 3.5  1998/04/22 00:53:02  brianp
- * fixed a few more IRIX compiler warnings
- *
- * Revision 3.4  1998/03/28 03:57:13  brianp
- * added CONST macro to fix IRIX compilation problems
- *
- * Revision 3.3  1998/03/28 02:03:22  brianp
- * hooked in MMX code
- *
- * Revision 3.2  1998/02/13 04:38:05  brianp
- * optimized blending functions called via BlendFunc function pointer
- *
- * Revision 3.1  1998/02/08 20:16:50  brianp
- * removed unneeded headers
- *
- * Revision 3.0  1998/01/31 20:47:46  brianp
- * initial rev
- *
- */
-
 
 #ifdef PC_HEADER
 #include "all.h"
 #else
+#ifndef XFree86Server
+#include <stdio.h>
 #include <assert.h>
 #include <stdlib.h>
+#else
+#include "GL/xf86glx.h"
+#endif
 #include "alphabuf.h"
-#include "asm_mmx.h"
 #include "blend.h"
 #include "context.h"
+#include "enums.h"
 #include "macros.h"
 #include "pb.h"
 #include "span.h"
@@ -69,13 +47,14 @@
 #endif
 
 
-
 void gl_BlendFunc( GLcontext *ctx, GLenum sfactor, GLenum dfactor )
 {
-   if (INSIDE_BEGIN_END(ctx)) {
-      gl_error( ctx, GL_INVALID_OPERATION, "glBlendFunc" );
-      return;
-   }
+   ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glBlendFunc");
+
+   if (MESA_VERBOSE & (VERBOSE_API|VERBOSE_TEXTURE))
+      fprintf(stderr, "glBlendFunc %s %s\n",
+	      gl_lookup_enum_by_nr(sfactor),
+	      gl_lookup_enum_by_nr(dfactor));
 
    switch (sfactor) {
       case GL_ZERO:
@@ -91,7 +70,7 @@ void gl_BlendFunc( GLcontext *ctx, GLenum sfactor, GLenum dfactor )
       case GL_ONE_MINUS_CONSTANT_COLOR:
       case GL_CONSTANT_ALPHA:
       case GL_ONE_MINUS_CONSTANT_ALPHA:
-         ctx->Color.BlendSrc = sfactor;
+         ctx->Color.BlendSrcRGB = ctx->Color.BlendSrcA = sfactor;
          break;
       default:
          gl_error( ctx, GL_INVALID_ENUM, "glBlendFunc(sfactor)" );
@@ -111,26 +90,138 @@ void gl_BlendFunc( GLcontext *ctx, GLenum sfactor, GLenum dfactor )
       case GL_ONE_MINUS_CONSTANT_COLOR:
       case GL_CONSTANT_ALPHA:
       case GL_ONE_MINUS_CONSTANT_ALPHA:
-         ctx->Color.BlendDst = dfactor;
+         ctx->Color.BlendDstRGB = ctx->Color.BlendDstA = dfactor;
          break;
       default:
          gl_error( ctx, GL_INVALID_ENUM, "glBlendFunc(dfactor)" );
          return;
    }
 
+   if (ctx->Driver.BlendFunc) {
+      (*ctx->Driver.BlendFunc)( ctx, sfactor, dfactor );
+   }
+
    ctx->Color.BlendFunc = NULL;
    ctx->NewState |= NEW_RASTER_OPS;
 }
 
 
+/* GL_INGR_blend_func_separate */
+void
+gl_BlendFuncSeparate( GLcontext *ctx, GLenum sfactorRGB, GLenum dfactorRGB,
+                      GLenum sfactorA, GLenum dfactorA )
+{
+   ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glBlendFuncSeparate");
+
+   if (MESA_VERBOSE & (VERBOSE_API|VERBOSE_TEXTURE))
+      fprintf(stderr, "glBlendFuncSeperate %s %s %s %s\n",
+	      gl_lookup_enum_by_nr(sfactorRGB),
+	      gl_lookup_enum_by_nr(dfactorRGB),
+	      gl_lookup_enum_by_nr(sfactorA),
+	      gl_lookup_enum_by_nr(dfactorA));
+
+   switch (sfactorRGB) {
+      case GL_ZERO:
+      case GL_ONE:
+      case GL_DST_COLOR:
+      case GL_ONE_MINUS_DST_COLOR:
+      case GL_SRC_ALPHA:
+      case GL_ONE_MINUS_SRC_ALPHA:
+      case GL_DST_ALPHA:
+      case GL_ONE_MINUS_DST_ALPHA:
+      case GL_SRC_ALPHA_SATURATE:
+      case GL_CONSTANT_COLOR:
+      case GL_ONE_MINUS_CONSTANT_COLOR:
+      case GL_CONSTANT_ALPHA:
+      case GL_ONE_MINUS_CONSTANT_ALPHA:
+         ctx->Color.BlendSrcRGB = sfactorRGB;
+         break;
+      default:
+         gl_error( ctx, GL_INVALID_ENUM, "glBlendFuncSeparate(sfactorRGB)" );
+         return;
+   }
+
+   switch (dfactorRGB) {
+      case GL_ZERO:
+      case GL_ONE:
+      case GL_SRC_COLOR:
+      case GL_ONE_MINUS_SRC_COLOR:
+      case GL_SRC_ALPHA:
+      case GL_ONE_MINUS_SRC_ALPHA:
+      case GL_DST_ALPHA:
+      case GL_ONE_MINUS_DST_ALPHA:
+      case GL_CONSTANT_COLOR:
+      case GL_ONE_MINUS_CONSTANT_COLOR:
+      case GL_CONSTANT_ALPHA:
+      case GL_ONE_MINUS_CONSTANT_ALPHA:
+         ctx->Color.BlendDstRGB = dfactorRGB;
+         break;
+      default:
+         gl_error( ctx, GL_INVALID_ENUM, "glBlendFuncSeparate(dfactorRGB)" );
+         return;
+   }
+
+   switch (sfactorA) {
+      case GL_ZERO:
+      case GL_ONE:
+      case GL_DST_COLOR:
+      case GL_ONE_MINUS_DST_COLOR:
+      case GL_SRC_ALPHA:
+      case GL_ONE_MINUS_SRC_ALPHA:
+      case GL_DST_ALPHA:
+      case GL_ONE_MINUS_DST_ALPHA:
+      case GL_SRC_ALPHA_SATURATE:
+      case GL_CONSTANT_COLOR:
+      case GL_ONE_MINUS_CONSTANT_COLOR:
+      case GL_CONSTANT_ALPHA:
+      case GL_ONE_MINUS_CONSTANT_ALPHA:
+         ctx->Color.BlendSrcA = sfactorA;
+         break;
+      default:
+         gl_error( ctx, GL_INVALID_ENUM, "glBlendFuncSeparate(sfactorA)" );
+         return;
+   }
+
+   switch (dfactorA) {
+      case GL_ZERO:
+      case GL_ONE:
+      case GL_SRC_COLOR:
+      case GL_ONE_MINUS_SRC_COLOR:
+      case GL_SRC_ALPHA:
+      case GL_ONE_MINUS_SRC_ALPHA:
+      case GL_DST_ALPHA:
+      case GL_ONE_MINUS_DST_ALPHA:
+      case GL_CONSTANT_COLOR:
+      case GL_ONE_MINUS_CONSTANT_COLOR:
+      case GL_CONSTANT_ALPHA:
+      case GL_ONE_MINUS_CONSTANT_ALPHA:
+         ctx->Color.BlendDstA = dfactorA;
+         break;
+      default:
+         gl_error( ctx, GL_INVALID_ENUM, "glBlendFuncSeparate(dfactorA)" );
+         return;
+   }
+
+   ctx->Color.BlendFunc = NULL;
+   ctx->NewState |= NEW_RASTER_OPS;
+
+   if (ctx->Driver.BlendFuncSeparate) {
+      (*ctx->Driver.BlendFuncSeparate)( ctx, sfactorRGB, dfactorRGB,
+					sfactorA, dfactorA );
+   }
+}
+
+
 
 /* This is really an extension function! */
 void gl_BlendEquation( GLcontext *ctx, GLenum mode )
 {
-   if (INSIDE_BEGIN_END(ctx)) {
-      gl_error( ctx, GL_INVALID_OPERATION, "glBlendEquation" );
-      return;
-   }
+   ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glBlendEquation");
+
+   if (MESA_VERBOSE & (VERBOSE_API|VERBOSE_TEXTURE))
+      fprintf(stderr, "glBlendEquation %s\n",
+	      gl_lookup_enum_by_nr(mode));
+
 
    switch (mode) {
       case GL_MIN_EXT:
@@ -158,6 +249,9 @@ void gl_BlendEquation( GLcontext *ctx, GLenum mode )
 
    ctx->Color.BlendFunc = NULL;
    ctx->NewState |= NEW_RASTER_OPS;
+   
+   if (ctx->Driver.BlendEquation)
+      ctx->Driver.BlendEquation( ctx, mode );
 }
 
 
@@ -165,10 +259,10 @@ void gl_BlendEquation( GLcontext *ctx, GLenum mode )
 void gl_BlendColor( GLcontext *ctx, GLclampf red, GLclampf green,
 		    GLclampf blue, GLclampf alpha )
 {
-   ctx->Color.BlendColor[0] = CLAMP( red,   0.0, 1.0 );
-   ctx->Color.BlendColor[1] = CLAMP( green, 0.0, 1.0 );
-   ctx->Color.BlendColor[2] = CLAMP( blue,  0.0, 1.0 );
-   ctx->Color.BlendColor[3] = CLAMP( alpha, 0.0, 1.0 );
+   ctx->Color.BlendColor[0] = CLAMP( red,   0.0F, 1.0F );
+   ctx->Color.BlendColor[1] = CLAMP( green, 0.0F, 1.0F );
+   ctx->Color.BlendColor[2] = CLAMP( blue,  0.0F, 1.0F );
+   ctx->Color.BlendColor[3] = CLAMP( alpha, 0.0F, 1.0F );
 }
 
 
@@ -181,8 +275,9 @@ static void blend_transparency( GLcontext *ctx, GLuint n, const GLubyte mask[],
 {
    GLuint i;
    ASSERT(ctx->Color.BlendEquation==GL_FUNC_ADD_EXT);
-   ASSERT(ctx->Color.BlendSrc==GL_SRC_ALPHA);
-   ASSERT(ctx->Color.BlendDst==GL_ONE_MINUS_SRC_ALPHA);
+   ASSERT(ctx->Color.BlendSrcRGB==GL_SRC_ALPHA);
+   ASSERT(ctx->Color.BlendDstRGB==GL_ONE_MINUS_SRC_ALPHA);
+   (void) ctx;
 
    for (i=0;i<n;i++) {
       if (mask[i]) {
@@ -206,10 +301,10 @@ static void blend_transparency( GLcontext *ctx, GLuint n, const GLubyte mask[],
             ASSERT(g <= 255);
             ASSERT(b <= 255);
             ASSERT(a <= 255);
-            rgba[i][RCOMP] = r;
-            rgba[i][GCOMP] = g;
-            rgba[i][BCOMP] = b;
-            rgba[i][ACOMP] = a;
+            rgba[i][RCOMP] = (GLubyte) r;
+            rgba[i][GCOMP] = (GLubyte) g;
+            rgba[i][BCOMP] = (GLubyte) b;
+            rgba[i][ACOMP] = (GLubyte) a;
          }
       }
    }
@@ -225,8 +320,8 @@ static void blend_add( GLcontext *ctx, GLuint n, const GLubyte mask[],
 {
    GLuint i;
    ASSERT(ctx->Color.BlendEquation==GL_FUNC_ADD_EXT);
-   ASSERT(ctx->Color.BlendSrc==GL_ONE);
-   ASSERT(ctx->Color.BlendDst==GL_ONE);
+   ASSERT(ctx->Color.BlendSrcRGB==GL_ONE);
+   ASSERT(ctx->Color.BlendDstRGB==GL_ONE);
    (void) ctx;
 
    for (i=0;i<n;i++) {
@@ -235,10 +330,10 @@ static void blend_add( GLcontext *ctx, GLuint n, const GLubyte mask[],
          GLint g = rgba[i][GCOMP] + dest[i][GCOMP];
          GLint b = rgba[i][BCOMP] + dest[i][BCOMP];
          GLint a = rgba[i][ACOMP] + dest[i][ACOMP];
-         rgba[i][RCOMP] = MIN2( r, 255 );
-         rgba[i][GCOMP] = MIN2( g, 255 );
-         rgba[i][BCOMP] = MIN2( b, 255 );
-         rgba[i][ACOMP] = MIN2( a, 255 );
+         rgba[i][RCOMP] = (GLubyte) MIN2( r, 255 );
+         rgba[i][GCOMP] = (GLubyte) MIN2( g, 255 );
+         rgba[i][BCOMP] = (GLubyte) MIN2( b, 255 );
+         rgba[i][ACOMP] = (GLubyte) MIN2( a, 255 );
       }
    }
 }
@@ -257,10 +352,10 @@ static void blend_min( GLcontext *ctx, GLuint n, const GLubyte mask[],
 
    for (i=0;i<n;i++) {
       if (mask[i]) {
-         rgba[i][RCOMP] = MIN2( rgba[i][RCOMP], dest[i][RCOMP] );
-         rgba[i][GCOMP] = MIN2( rgba[i][GCOMP], dest[i][GCOMP] );
-         rgba[i][BCOMP] = MIN2( rgba[i][BCOMP], dest[i][BCOMP] );
-         rgba[i][ACOMP] = MIN2( rgba[i][ACOMP], dest[i][ACOMP] );
+         rgba[i][RCOMP] = (GLubyte) MIN2( rgba[i][RCOMP], dest[i][RCOMP] );
+         rgba[i][GCOMP] = (GLubyte) MIN2( rgba[i][GCOMP], dest[i][GCOMP] );
+         rgba[i][BCOMP] = (GLubyte) MIN2( rgba[i][BCOMP], dest[i][BCOMP] );
+         rgba[i][ACOMP] = (GLubyte) MIN2( rgba[i][ACOMP], dest[i][ACOMP] );
       }
    }
 }
@@ -279,10 +374,10 @@ static void blend_max( GLcontext *ctx, GLuint n, const GLubyte mask[],
 
    for (i=0;i<n;i++) {
       if (mask[i]) {
-         rgba[i][RCOMP] = MAX2( rgba[i][RCOMP], dest[i][RCOMP] );
-         rgba[i][GCOMP] = MAX2( rgba[i][GCOMP], dest[i][GCOMP] );
-         rgba[i][BCOMP] = MAX2( rgba[i][BCOMP], dest[i][BCOMP] );
-         rgba[i][ACOMP] = MAX2( rgba[i][ACOMP], dest[i][ACOMP] );
+         rgba[i][RCOMP] = (GLubyte) MAX2( rgba[i][RCOMP], dest[i][RCOMP] );
+         rgba[i][GCOMP] = (GLubyte) MAX2( rgba[i][GCOMP], dest[i][GCOMP] );
+         rgba[i][BCOMP] = (GLubyte) MAX2( rgba[i][BCOMP], dest[i][BCOMP] );
+         rgba[i][ACOMP] = (GLubyte) MAX2( rgba[i][ACOMP], dest[i][ACOMP] );
       }
    }
 }
@@ -304,10 +399,10 @@ static void blend_modulate( GLcontext *ctx, GLuint n, const GLubyte mask[],
          GLint g = (rgba[i][GCOMP] * dest[i][GCOMP]) >> 8;
          GLint b = (rgba[i][BCOMP] * dest[i][BCOMP]) >> 8;
          GLint a = (rgba[i][ACOMP] * dest[i][ACOMP]) >> 8;
-         rgba[i][RCOMP] = r;
-         rgba[i][GCOMP] = g;
-         rgba[i][BCOMP] = b;
-         rgba[i][ACOMP] = a;
+         rgba[i][RCOMP] = (GLubyte) r;
+         rgba[i][GCOMP] = (GLubyte) g;
+         rgba[i][BCOMP] = (GLubyte) b;
+         rgba[i][ACOMP] = (GLubyte) a;
       }
    }
 }
@@ -350,123 +445,208 @@ static void blend_general( GLcontext *ctx, GLuint n, const GLubyte mask[],
          Bd = dest[i][BCOMP];
          Ad = dest[i][ACOMP];
 
-         /* Source scaling */
-         switch (ctx->Color.BlendSrc) {
+         /* Source RGB factor */
+         switch (ctx->Color.BlendSrcRGB) {
             case GL_ZERO:
-               sR = sG = sB = sA = 0.0F;
+               sR = sG = sB = 0.0F;
                break;
             case GL_ONE:
-               sR = sG = sB = sA = 1.0F;
+               sR = sG = sB = 1.0F;
                break;
             case GL_DST_COLOR:
                sR = (GLfloat) Rd * rscale;
                sG = (GLfloat) Gd * gscale;
                sB = (GLfloat) Bd * bscale;
-               sA = (GLfloat) Ad * ascale;
                break;
             case GL_ONE_MINUS_DST_COLOR:
                sR = 1.0F - (GLfloat) Rd * rscale;
                sG = 1.0F - (GLfloat) Gd * gscale;
                sB = 1.0F - (GLfloat) Bd * bscale;
-               sA = 1.0F - (GLfloat) Ad * ascale;
                break;
             case GL_SRC_ALPHA:
-               sR = sG = sB = sA = (GLfloat) As * ascale;
+               sR = sG = sB = (GLfloat) As * ascale;
                break;
             case GL_ONE_MINUS_SRC_ALPHA:
-               sR = sG = sB = sA = (GLfloat) 1.0F - (GLfloat) As * ascale;
+               sR = sG = sB = (GLfloat) 1.0F - (GLfloat) As * ascale;
                break;
             case GL_DST_ALPHA:
-               sR = sG = sB = sA =(GLfloat) Ad * ascale;
+               sR = sG = sB = (GLfloat) Ad * ascale;
                break;
             case GL_ONE_MINUS_DST_ALPHA:
-               sR = sG = sB = sA = 1.0F - (GLfloat) Ad * ascale;
+               sR = sG = sB = 1.0F - (GLfloat) Ad * ascale;
                break;
             case GL_SRC_ALPHA_SATURATE:
-               if (As < 1.0F - (GLfloat) Ad * ascale) {
+               if (As * ascale < 1.0F - (GLfloat) Ad * ascale) {
                   sR = sG = sB = (GLfloat) As * ascale;
                }
                else {
                   sR = sG = sB = 1.0F - (GLfloat) Ad * ascale;
                }
-               sA = 1.0;
                break;
             case GL_CONSTANT_COLOR:
                sR = ctx->Color.BlendColor[0];
                sG = ctx->Color.BlendColor[1];
                sB = ctx->Color.BlendColor[2];
-               sA = ctx->Color.BlendColor[3];
                break;
             case GL_ONE_MINUS_CONSTANT_COLOR:
                sR = 1.0F - ctx->Color.BlendColor[0];
                sG = 1.0F - ctx->Color.BlendColor[1];
                sB = 1.0F - ctx->Color.BlendColor[2];
-               sA = 1.0F - ctx->Color.BlendColor[3];
                break;
             case GL_CONSTANT_ALPHA:
-               sR = sG = sB = sA = ctx->Color.BlendColor[3];
+               sR = sG = sB = ctx->Color.BlendColor[3];
                break;
             case GL_ONE_MINUS_CONSTANT_ALPHA:
-               sR = sG = sB = sA = 1.0F - ctx->Color.BlendColor[3];
+               sR = sG = sB = 1.0F - ctx->Color.BlendColor[3];
                break;
             default:
                /* this should never happen */
-               gl_problem(ctx, "Bad blend source factor in do_blend");
+               gl_problem(ctx, "Bad blend source RGB factor in do_blend");
+	       return;
          }
 
-         /* Dest scaling */
-         switch (ctx->Color.BlendDst) {
+         /* Source Alpha factor */
+         switch (ctx->Color.BlendSrcA) {
             case GL_ZERO:
-               dR = dG = dB = dA = 0.0F;
+               sA = 0.0F;
                break;
             case GL_ONE:
-               dR = dG = dB = dA = 1.0F;
+               sA = 1.0F;
+               break;
+            case GL_DST_COLOR:
+               sA = (GLfloat) Ad * ascale;
+               break;
+            case GL_ONE_MINUS_DST_COLOR:
+               sA = 1.0F - (GLfloat) Ad * ascale;
+               break;
+            case GL_SRC_ALPHA:
+               sA = (GLfloat) As * ascale;
+               break;
+            case GL_ONE_MINUS_SRC_ALPHA:
+               sA = (GLfloat) 1.0F - (GLfloat) As * ascale;
+               break;
+            case GL_DST_ALPHA:
+               sA =(GLfloat) Ad * ascale;
+               break;
+            case GL_ONE_MINUS_DST_ALPHA:
+               sA = 1.0F - (GLfloat) Ad * ascale;
+               break;
+            case GL_SRC_ALPHA_SATURATE:
+               sA = 1.0;
+               break;
+            case GL_CONSTANT_COLOR:
+               sA = ctx->Color.BlendColor[3];
+               break;
+            case GL_ONE_MINUS_CONSTANT_COLOR:
+               sA = 1.0F - ctx->Color.BlendColor[3];
+               break;
+            case GL_CONSTANT_ALPHA:
+               sA = ctx->Color.BlendColor[3];
+               break;
+            case GL_ONE_MINUS_CONSTANT_ALPHA:
+               sA = 1.0F - ctx->Color.BlendColor[3];
+               break;
+            default:
+               /* this should never happen */
+               sA = 0.0F;
+               gl_problem(ctx, "Bad blend source A factor in do_blend");
+         }
+
+         /* Dest RGB factor */
+         switch (ctx->Color.BlendDstRGB) {
+            case GL_ZERO:
+               dR = dG = dB = 0.0F;
+               break;
+            case GL_ONE:
+               dR = dG = dB = 1.0F;
                break;
             case GL_SRC_COLOR:
                dR = (GLfloat) Rs * rscale;
                dG = (GLfloat) Gs * gscale;
                dB = (GLfloat) Bs * bscale;
-               dA = (GLfloat) As * ascale;
                break;
             case GL_ONE_MINUS_SRC_COLOR:
                dR = 1.0F - (GLfloat) Rs * rscale;
                dG = 1.0F - (GLfloat) Gs * gscale;
                dB = 1.0F - (GLfloat) Bs * bscale;
-               dA = 1.0F - (GLfloat) As * ascale;
                break;
             case GL_SRC_ALPHA:
-               dR = dG = dB = dA = (GLfloat) As * ascale;
+               dR = dG = dB = (GLfloat) As * ascale;
                break;
             case GL_ONE_MINUS_SRC_ALPHA:
-               dR = dG = dB = dA = (GLfloat) 1.0F - (GLfloat) As * ascale;
+               dR = dG = dB = (GLfloat) 1.0F - (GLfloat) As * ascale;
                break;
             case GL_DST_ALPHA:
-               dR = dG = dB = dA = (GLfloat) Ad * ascale;
+               dR = dG = dB = (GLfloat) Ad * ascale;
                break;
             case GL_ONE_MINUS_DST_ALPHA:
-               dR = dG = dB = dA = 1.0F - (GLfloat) Ad * ascale;
+               dR = dG = dB = 1.0F - (GLfloat) Ad * ascale;
                break;
             case GL_CONSTANT_COLOR:
                dR = ctx->Color.BlendColor[0];
                dG = ctx->Color.BlendColor[1];
                dB = ctx->Color.BlendColor[2];
-               dA = ctx->Color.BlendColor[3];
                break;
             case GL_ONE_MINUS_CONSTANT_COLOR:
                dR = 1.0F - ctx->Color.BlendColor[0];
                dG = 1.0F - ctx->Color.BlendColor[1];
                dB = 1.0F - ctx->Color.BlendColor[2];
-               dA = 1.0F - ctx->Color.BlendColor[3];
                break;
             case GL_CONSTANT_ALPHA:
-               dR = dG = dB = dA = ctx->Color.BlendColor[3];
+               dR = dG = dB = ctx->Color.BlendColor[3];
                break;
             case GL_ONE_MINUS_CONSTANT_ALPHA:
-               dR = dG = dB = dA = 1.0F - ctx->Color.BlendColor[3] * ascale;
+               dR = dG = dB = 1.0F - ctx->Color.BlendColor[3] * ascale;
                break;
             default:
                /* this should never happen */
-               gl_problem(ctx, "Bad blend dest factor in do_blend");
+               dR = dG = dB = 0.0F;
+               gl_problem(ctx, "Bad blend dest RGB factor in do_blend");
+         }
+
+         /* Dest Alpha factor */
+         switch (ctx->Color.BlendDstA) {
+            case GL_ZERO:
+               dA = 0.0F;
+               break;
+            case GL_ONE:
+               dA = 1.0F;
+               break;
+            case GL_SRC_COLOR:
+               dA = (GLfloat) As * ascale;
+               break;
+            case GL_ONE_MINUS_SRC_COLOR:
+               dA = 1.0F - (GLfloat) As * ascale;
+               break;
+            case GL_SRC_ALPHA:
+               dA = (GLfloat) As * ascale;
+               break;
+            case GL_ONE_MINUS_SRC_ALPHA:
+               dA = (GLfloat) 1.0F - (GLfloat) As * ascale;
+               break;
+            case GL_DST_ALPHA:
+               dA = (GLfloat) Ad * ascale;
+               break;
+            case GL_ONE_MINUS_DST_ALPHA:
+               dA = 1.0F - (GLfloat) Ad * ascale;
+               break;
+            case GL_CONSTANT_COLOR:
+               dA = ctx->Color.BlendColor[3];
+               break;
+            case GL_ONE_MINUS_CONSTANT_COLOR:
+               dA = 1.0F - ctx->Color.BlendColor[3];
+               break;
+            case GL_CONSTANT_ALPHA:
+               dA = ctx->Color.BlendColor[3];
+               break;
+            case GL_ONE_MINUS_CONSTANT_ALPHA:
+               dA = 1.0F - ctx->Color.BlendColor[3] * ascale;
+               break;
+            default:
+               /* this should never happen */
+               dA = 0.0F;
+               gl_problem(ctx, "Bad blend dest A factor in do_blend");
+	       return;
          }
 
          /* Due to round-off problems we have to clamp against zero. */
@@ -508,43 +688,64 @@ static void blend_general( GLcontext *ctx, GLuint n, const GLubyte mask[],
             b = Bd * dB - Bs * sB;
             a = Ad * dA - As * sA;
          }
+         else {
+            /* should never get here */
+            r = g = b = a = 0.0F;  /* silence uninitialized var warning */
+            gl_problem(ctx, "unexpected BlendEquation in blend_general()");
+         }
 
          /* final clamping */
-         rgba[i][RCOMP] = (GLint) CLAMP( r, 0.0F, 255.0F );
-         rgba[i][GCOMP] = (GLint) CLAMP( g, 0.0F, 255.0F );
-         rgba[i][BCOMP] = (GLint) CLAMP( b, 0.0F, 255.0F );
-         rgba[i][ACOMP] = (GLint) CLAMP( a, 0.0F, 255.0F );
+         rgba[i][RCOMP] = (GLubyte) (GLint) CLAMP( r, 0.0F, 255.0F );
+         rgba[i][GCOMP] = (GLubyte) (GLint) CLAMP( g, 0.0F, 255.0F );
+         rgba[i][BCOMP] = (GLubyte) (GLint) CLAMP( b, 0.0F, 255.0F );
+         rgba[i][ACOMP] = (GLubyte) (GLint) CLAMP( a, 0.0F, 255.0F );
       }
    }
 }
 
 
 
+#if defined(USE_MMX_ASM)
+#include "X86/mmx.h"
+#include "X86/common_x86asm.h"
+#endif
+
+
 /*
  * Analyze current blending parameters to pick fastest blending function.
  * Result: the ctx->Color.BlendFunc pointer is updated.
  */
-static void set_blend_function(GLcontext *ctx)
+static void set_blend_function( GLcontext *ctx )
 {
-   GLenum eq = ctx->Color.BlendEquation;
-   GLenum src = ctx->Color.BlendSrc;
-   GLenum dst = ctx->Color.BlendDst;
+   const GLenum eq = ctx->Color.BlendEquation;
+   const GLenum srcRGB = ctx->Color.BlendSrcRGB;
+   const GLenum dstRGB = ctx->Color.BlendDstRGB;
+   const GLenum srcA = ctx->Color.BlendSrcA;
+   const GLenum dstA = ctx->Color.BlendDstA;
 
-   if (eq==GL_FUNC_ADD_EXT && src==GL_SRC_ALPHA && dst==GL_ONE_MINUS_SRC_ALPHA) {
-#ifdef MMX
-      ctx->Color.BlendFunc = gl_mmx_blend_transparency;
-#else
-      ctx->Color.BlendFunc = blend_transparency;
+#if defined(USE_MMX_ASM)
+   /* Hmm.  A table here would have 12^4 == way too many entries.
+    * Provide a hook for MMX instead.
+    */
+   if (gl_x86_cpu_features & GL_CPU_MMX) {
+      gl_mmx_set_blend_function (ctx);
+   } else
 #endif
+   if (srcRGB != srcA || dstRGB != dstA) {
+      ctx->Color.BlendFunc = blend_general;
    }
-   else if (eq==GL_FUNC_ADD_EXT && src==GL_ONE && dst==GL_ONE) {
+   else if (eq==GL_FUNC_ADD_EXT && srcRGB==GL_SRC_ALPHA
+       && dstRGB==GL_ONE_MINUS_SRC_ALPHA) {
+      ctx->Color.BlendFunc = blend_transparency;
+   }
+   else if (eq==GL_FUNC_ADD_EXT && srcRGB==GL_ONE && dstRGB==GL_ONE) {
       ctx->Color.BlendFunc = blend_add;
    }
    else if (((eq==GL_FUNC_ADD_EXT || eq==GL_FUNC_REVERSE_SUBTRACT_EXT)
-             && (src==GL_ZERO && dst==GL_SRC_COLOR))
+             && (srcRGB==GL_ZERO && dstRGB==GL_SRC_COLOR))
             ||
             ((eq==GL_FUNC_ADD_EXT || eq==GL_FUNC_SUBTRACT_EXT)
-             && (src==GL_DST_COLOR && dst==GL_ZERO))) {
+             && (srcRGB==GL_DST_COLOR && dstRGB==GL_ZERO))) {
       ctx->Color.BlendFunc = blend_modulate;
    }
    else if (eq==GL_MIN_EXT) {
@@ -583,7 +784,7 @@ void gl_blend_span( GLcontext *ctx, GLuint n, GLint x, GLint y,
    if (!ctx->Color.BlendFunc)
       set_blend_function(ctx);
 
-   (*ctx->Color.BlendFunc)( ctx, n, mask, rgba, dest );
+   (*ctx->Color.BlendFunc)( ctx, n, mask, rgba, (const GLubyte (*)[4])dest );
 }
 
 
@@ -610,19 +811,12 @@ void gl_blend_pixels( GLcontext *ctx,
 
    /* Read pixels from current color buffer */
    (*ctx->Driver.ReadRGBAPixels)( ctx, n, x, y, dest, mask );
-
    if (ctx->RasterMask & ALPHABUF_BIT) {
       gl_read_alpha_pixels( ctx, n, x, y, dest, mask );
    }
-   else {
-      GLuint i;
-      for (i=0; i<n; i++) {
-         dest[i][ACOMP] = 255;
-      }
-   }
 
    if (!ctx->Color.BlendFunc)
       set_blend_function(ctx);
 
-   (*ctx->Color.BlendFunc)( ctx, n, mask, rgba, dest );
+   (*ctx->Color.BlendFunc)( ctx, n, mask, rgba, (const GLubyte (*)[4])dest );
 }
