@@ -1,42 +1,49 @@
-/* $Id: mmath.c,v 3.0 1998/01/31 20:59:27 brianp Exp $ */
+/* $Id: mmath.c,v 1.4 1999/11/08 07:36:44 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.0
- * Copyright (C) 1995-1998  Brian Paul
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Version:  3.1
+ * 
+ * Copyright (C) 1999  Brian Paul   All Rights Reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
+/* $XFree86: xc/lib/GL/mesa/src/mmath.c,v 1.2 1999/04/04 00:20:28 dawes Exp $ */
+
 
 
-/*
- * $Log: mmath.c,v $
- * Revision 3.0  1998/01/31 20:59:27  brianp
- * initial rev
- *
- */
 
 
 #ifdef PC_HEADER
 #include "all.h"
 #else
+#ifndef XFree86Server
+#include <math.h>
+#else
+#include "GL/xf86glx.h"
+#endif
 #include "GL/gl.h"
 #include "mmath.h"
 #endif
 
 
+static int in_fast_math;
+
 
 /*
  * A High Speed, Low Precision Square Root
@@ -91,7 +98,9 @@ static void init_sqrt(void)
       f = sqrt(f);
       sqrttab[i+0x80] = (*fi & 0x7fffff) >> 16;
    }
-#endif /*FAST_MATH*/
+#else
+   (void) sqrttab;  /* silence compiler warning - unused var */
+#endif
 }
 
 
@@ -126,7 +135,18 @@ float gl_sqrt( float x )
 #endif
 }
 
+float gl_ubyte_to_float_color_tab[256];
+float gl_ubyte_to_float_255_color_tab[256];
 
+static void
+init_ubyte_color_tab(void)
+{
+   int i;
+   for (i = 0 ; i < 256 ; i++) {
+      gl_ubyte_to_float_color_tab[i] = (float) i * (1.0/255.0);
+      gl_ubyte_to_float_255_color_tab[i] = (float) i;
+   }
+}
 
 /*
  * Initialize tables, etc for fast math functions.
@@ -137,8 +157,9 @@ void gl_init_math(void)
 
    if (!initialized) {
       init_sqrt();
-
+      init_ubyte_color_tab();
 
       initialized = GL_TRUE;
+      in_fast_math = 0;
    }
 }
