@@ -1,5 +1,5 @@
 /* -*- mode: C; tab-width:8; c-basic-offset:8 -*- */
-/* $Id: types.h,v 1.18.2.3 1999/12/04 21:13:43 brianp Exp $ */
+/* $Id: types.h,v 1.18.2.9 2000/02/21 22:49:24 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -124,7 +124,7 @@
 /*
  * Some forward type declarations
  */
-struct HashTable;
+struct _mesa_HashTable;
 
 typedef struct gl_visual GLvisual;
 
@@ -1046,6 +1046,9 @@ struct gl_texture_attrib {
 	GLuint PaletteSize;
 	GLenum PaletteIntFormat;
 	GLenum PaletteFormat;
+
+	/* GL_EXT_texture_env_add */
+	GLboolean ExtAddEnv;	/* is the extension enabled? */
 };
 
 
@@ -1403,8 +1406,8 @@ struct gl_evaluators {
  */
 struct gl_shared_state {
    GLint RefCount;			   /* Reference count */
-   struct HashTable *DisplayList;	   /* Display lists hash table */
-   struct HashTable *TexObjects;	   /* Texture objects hash table */
+   struct _mesa_HashTable *DisplayList;	   /* Display lists hash table */
+   struct _mesa_HashTable *TexObjects;	   /* Texture objects hash table */
    struct gl_texture_object *TexObjectList;/* Linked list of texture objects */
    struct gl_texture_object *DirtyTexObjList; /* List of dirty tex objects */
 
@@ -1883,13 +1886,13 @@ struct gl_context {
 	/* Modelview matrix and stack */
 	GLmatrix ModelView;	
 	GLuint ModelViewStackDepth;
-	GLmatrix ModelViewStack[MAX_MODELVIEW_STACK_DEPTH];
+	GLmatrix ModelViewStack[MAX_MODELVIEW_STACK_DEPTH - 1];
 
 	/* Projection matrix and stack */
 	GLmatrix ProjectionMatrix;
 	GLuint ProjectionStackDepth;
-	GLmatrix ProjectionStack[MAX_PROJECTION_STACK_DEPTH];
-	GLfloat NearFarStack[MAX_PROJECTION_STACK_DEPTH][2];
+	GLmatrix ProjectionStack[MAX_PROJECTION_STACK_DEPTH - 1];
+	GLfloat NearFarStack[MAX_PROJECTION_STACK_DEPTH - 1][2];
 
         /* Combined modelview and projection matrix */
         GLmatrix ModelProjectMatrix;
@@ -1901,7 +1904,7 @@ struct gl_context {
 	/* Texture matrix and stack */
 	GLmatrix TextureMatrix[MAX_TEXTURE_UNITS];
 	GLuint TextureStackDepth[MAX_TEXTURE_UNITS];
-	GLmatrix TextureStack[MAX_TEXTURE_UNITS][MAX_TEXTURE_STACK_DEPTH];
+	GLmatrix TextureStack[MAX_TEXTURE_UNITS][MAX_TEXTURE_STACK_DEPTH - 1];
 
 	/* Display lists */
 	GLuint CallDepth;	/* Current recursion calling depth */
@@ -2089,7 +2092,9 @@ enum _verbose {
 	VERBOSE_API             = 0x40,
 	VERBOSE_TRIANGLE_CHECKS = 0x80,
 	VERBOSE_CULL            = 0x100,
-	VERBOSE_DISPLAY_LIST    = 0x200
+	VERBOSE_DISPLAY_LIST    = 0x200,
+	VERBOSE_LIGHTING        = 0x400,
+	VERBOSE_EXTENSIONS      = 0x800
 }; 
 
 
